import os, sysconfig, darkdetect



def add():
    """Adds the Init-Film shortcut to the Windows context menu."""


    osname = os.name
    if osname != 'nt':
        print("This feature currently only works on Windows!")
        exit()
    else:
        import winreg


    try:
        python_scripts = sysconfig.get_path("scripts")
        python_sitepackages = sysconfig.get_path("purelib")
        windows_theme = darkdetect.theme()

        location = winreg.HKEY_CURRENT_USER
        shell = winreg.CreateKey(location, "Software\\Classes\\Directory\\Background\\shell")
        initfilm = winreg.CreateKey(shell, "Init-Film")
        command = winreg.CreateKey(initfilm, "command")

        winreg.SetValueEx(initfilm, "", 0, winreg.REG_SZ, "Start Init-Film Project") # Shortcut name
        winreg.SetValueEx(command, "", 0, winreg.REG_SZ, f"{python_scripts}/init-film.exe") # Shortcut executable path

        if windows_theme == 'Dark':
            winreg.SetValueEx(initfilm, "Icon", 0, winreg.REG_SZ, f"{python_sitepackages}/initfilm/Init-Film_Icon_White.ico")
        else:
            winreg.SetValueEx(initfilm, "Icon", 0, winreg.REG_SZ, f"{python_sitepackages}/initfilm/Init-Film_Icon_Black.ico")
        
        print("Successfully added Init-Film to context menu!")


    except Exception as err:
        if str(err).__contains__('WinError 5'):
            print(err, "\n\nAre you administrator?")
        else:
            print(err)



def remove():
    """Removes the Init-Film shortcut to the Windows context menu."""

    
    osname = os.name
    if osname != 'nt':
        print("This feature currently only works on Windows!")
        exit()
    else:
        import winreg


    try:
        location = winreg.HKEY_CURRENT_USER
        shell = winreg.CreateKey(location, "Software\\Classes\\Directory\\Background\\shell")
        initfilm = winreg.CreateKey(shell, "Init-Film")
        command = winreg.CreateKey(initfilm, "command")

        winreg.DeleteValue(command, "")
        winreg.DeleteValue(initfilm, "")
        winreg.DeleteValue(initfilm, "Icon")

        winreg.DeleteKey(command, "")
        winreg.CloseKey(command)
        winreg.DeleteKey(initfilm, "")
        winreg.CloseKey(initfilm)
        
        print("Successfully removed Init-Film from context menu!")


    except Exception as err:
        if str(err).__contains__('WinError 5'):
            print(err, "\n\nAre you administrator?")
        elif str(err).__contains__('WinError 2'):
            print(err, "\n\nThis probably means the shortcut has already been removed, or hasn't been installed correctly...")
        else:
            print(err)


import os, sysconfig, shutil
import initfilm.config



python_sitepackages = sysconfig.get_path("purelib")
template_path = initfilm.config.get("templates", "path")

if template_path == "python":
    template_path = f"{python_sitepackages}/initfilm/templates"
    os.makedirs(template_path, exist_ok=True)
else:
    template_path = template_path
    os.makedirs(template_path, exist_ok=True)



def clearConsole():
    """Clears the console."""

    osname = os.name
    if osname == 'nt':
        _ = os.system('cls') # For Windows
    else:
        _ = os.system('clear') # For macOS & Linux



def check(relative_source_path:str) -> bool:
    """Checks if source folder contains template files.
    Args:
        relative_source_path (str): Path to template files, relative to `template_path` (ex. "ASSETS/LOGOS")
    Returns:
        result (bool): `True` if folder contains files, `False` if it doesn't."""

    source_path = os.path.join(template_path, relative_source_path)

    try:
        files = [f for f in os.listdir(source_path) if os.path.isfile(os.path.join(source_path, f)) and f != ".DS_Store"] # Lists all contents in source_path & excludes folders and .DS_Store files

        if files == []:
            return False
        else:
            return True

    except:
        return False



def copy(relative_source_path:str, destination_path:str, destination_name:str):
    """
    Lists all template files in source folder, let's user select files & copies files to destination.
    Args:
        relative_source_path (str): Path to template files, relative to `template_path` (ex. "ASSETS/LOGOS").
        destination_path (str): Full path to destination folder.
        destination_name (str): Name of the destination folder, used in the selection screen (ex. 1. PROJECT FILES).
    """

    source_path = os.path.join(template_path, relative_source_path)
    files = [f for f in os.listdir(source_path) if os.path.isfile(os.path.join(source_path, f)) and f != ".DS_Store"] # Lists all contents in source_path & excludes folders and .DS_Store files

    clearConsole()
    print(f'Select needed files for {destination_name}\n')
    for id, template in enumerate(files, start=1):
        print(f'    {id}) {template}')
    print('\nType numbers to select (separate by SPACE)')
    selection = input('$ ').split()

    for id in selection:
        shutil.copy2(f"{source_path}/{files[int(id) - 1]}", destination_path)


import sys, importlib.metadata
import initfilm.main
import initfilm.shortcut
import initfilm.config
import initfilm.templates


def main():
    version = importlib.metadata.version("init-film")


    if len(sys.argv) > 1:
        argument = sys.argv[1]

        if argument == "--help" or argument == "-h":
            print(f'''\
------------------
Init-Film v{version}
By Andreas Delabie
------------------

Usage: init-film [option]
e.g. init-film --set-number-style double

Options:
    --set-prefix <option>           Set prefix to be 'visible' or 'hidden'
    --set-number-style <style>      Set number style to default (1. PROJECT FILES) or double (01. PROJECT FILES)
    --set-separator-style <style>   Set separator style to dot (default), underscore, parenthesis or space

    --set-templates-path <path>     Set templates path (source path) to 'python' or custom path. ALWAYS USE FORWARD SLASHES!
    --show-templates-path           Print templates path

    --show-config                   Print current configuration

    --add-shortcut                  Adds the Init-Film shortcut to the Windows context menu (right click menu)
    --remove-shortcut               Removes the Init-Film shortcut to the Windows context menu (right click menu)

    -v --version                    Print current version
    -h --help                       Shows this screen
''')

        elif argument == "--set-prefix":
            if len(sys.argv) > 2:
                initfilm.config.setPrefixVisibility(sys.argv[2])
            else:
                print("Please specify if you want the prefix to be 'visible' or 'hidden'")

        elif argument == "--set-number-style":
            if len(sys.argv) > 2:
                initfilm.config.setNumberStyle(sys.argv[2])
            else:
                print("Please specify a valid number style like 'default' or 'double'.")
        
        elif argument == "--set-separator-style":
            if len(sys.argv) > 2:
                initfilm.config.setSeparatorStyle(sys.argv[2])
            else:
                print("Please specify a valid separator style like 'dot', 'underscore', 'parenthesis' or 'space'.")
        
        elif argument == "--set-templates-path":
            if len(sys.argv) > 2:
                initfilm.config.set("templates", "path", sys.argv[2])
            else:
                print("Please specify a templates path like 'python' or a custom path (ex. C:/Users/Spielberg/Videos/templates).")
        
        elif argument == "--show-templates-path":
            print(initfilm.templates.template_path)
        
        elif argument == "--show-config":
            initfilm.config.show()

        elif argument == "--add-shortcut":
            initfilm.shortcut.add()

        elif argument == "--remove-shortcut":
            initfilm.shortcut.remove()
        
        elif argument == "--version" or argument == "-v":
            print(version)

        else:
            print("Unknown command, use --help or -h for help")


    else:
        initfilm.main.createFolder_level0()


import os
import initfilm.config
import initfilm.templates



def clearConsole():
    """Clears the console."""

    osname = os.name
    if osname == 'nt':
        _ = os.system('cls') # For Windows
    else:
        _ = os.system('clear') # For macOS & Linux



def prefix(number_input:int) -> str:
    """Returns a styled prefix based on the values in the config file.
    Args:
        number_input (int): The prefix number.
    Returns:
        prefix (str): Styled prefix with a number and separator."""

    try:
        visible = initfilm.config.get("prefix", "visibility")
    except:
        initfilm.config.setPrefixVisibility("visible")
        visible = initfilm.config.get("prefix", "visibility")
    number_style = initfilm.config.get("prefix", "number_style")
    separator_style = initfilm.config.get("prefix", "separator_style")

    if number_style == "double":
        number_output = f"{number_input:02d}"
    else:
        number_output = number_input
    
    if separator_style == "underscore":
        separator = "_"
    elif separator_style == "parenthesis":
        separator = ") "
    elif separator_style == "space":
        separator = " "
    else:
        separator = ". "
    
    if visible == "hidden":
        return ""
    else:
        return f"{number_output}{separator}"



def createFolder_level0():
    global projectDirectory, workingTitle

    clearConsole()
    print('Enter project name:')

    workingTitle = input('$ ')
    currentDirectory = os.getcwd()
    projectDirectory = os.path.join(currentDirectory, workingTitle)

    os.mkdir(projectDirectory)
    if initfilm.templates.check("") == True:
        initfilm.templates.copy("", projectDirectory, workingTitle)

    createFolder_level1_project()



def createFolder_level1(number:int, name:str):
    global level1_folder_name
    level1_folder_name = f'{prefix(number)}{name}'
    level1_folder_directory = os.path.join(projectDirectory, level1_folder_name)

    os.mkdir(level1_folder_directory)
    if initfilm.templates.check(name) == True:
        initfilm.templates.copy(name, level1_folder_directory, level1_folder_name)



def createFolder_level1_project():
    clearConsole()
    print(f'''Select needed level 1 folders for {workingTitle}

    1) PROJECT FILES
    2) ASSETS
    3) FOOTAGE
    4) AUDIO
    5) EXPORT
    6) DOCUMENTS
    7) MARKETING

Type numbers to select or type custom folder names (separate by SPACE)''')

    level1_project_selection = input('$ ').split()
    global level1_project_folderNumber
    level1_project_folderNumber = 1

    for string in level1_project_selection:
        if string == '1':
            createFolder_level1(level1_project_folderNumber, 'PROJECT FILES')
            createFolder_level2_projectFiles()
        elif string == '2':
            createFolder_level1(level1_project_folderNumber, 'ASSETS')
            createFolder_level2_assets()
        elif string == '3':
            createFolder_level1(level1_project_folderNumber, 'FOOTAGE')
            createFolder_level2_footage()
        elif string == '4':
            createFolder_level1(level1_project_folderNumber, 'AUDIO')
            createFolder_level2_audio()
        elif string == '5':
            createFolder_level1(level1_project_folderNumber, 'EXPORT')
            createFolder_level2_export()
        elif string == '6':
            createFolder_level1(level1_project_folderNumber, 'DOCUMENTS')
            createFolder_level2_documents()
        elif string == '7':
            createFolder_level1(level1_project_folderNumber, 'MARKETING')
            createFolder_level2_marketing()
        else:
            createFolder_level1(level1_project_folderNumber, string.upper())
            createFolder_level2_custom(f'{prefix(level1_project_folderNumber)}{string.upper()}', string.upper())
        level1_project_folderNumber = level1_project_folderNumber + 1



def createFolder_level2(parent:str, number:int, name:str, relative_template_path:str):
    global level2_folder_name
    level2_folder_name = f'{prefix(number)}{name}'
    level2_folder_directory = os.path.join(projectDirectory, parent, level2_folder_name)

    os.mkdir(level2_folder_directory)
    if initfilm.templates.check(relative_template_path) == True:
        initfilm.templates.copy(relative_template_path, level2_folder_directory, level2_folder_name)



def createFolder_level2_projectFiles():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}:

    EDITING:
        1) PREMIERE PRO
        2) DAVINCI RESOLVE
        3) MEDIA COMPOSER
        4) FINAL CUT PRO
        5) SONY VEGAS
        6) HITFILM EXPRESS

    GRAPHICS/VFX:
        7) AFTER EFFECTS
        8) PHOTOSHOP
        9) ILLUSTRATOR
        10) BLENDER
        11) CINEMA 4D
        12) MAYA
        13) NUKE
        14) HOUDINI
        15) ZBRUSH
        16) AFFINITY DESIGNER

    AUDIO/MUSIC:
        17) PRO TOOLS
        18) ABLETON
        19) CUBASE
        20) REAPER
        21) IZOTOPE RX
        22) AUDITION
        23) AUDACITY
        24) LOGIC PRO
        25) FL STUDIO

Type numbers to select or type custom folder names (separate by SPACE)''')

    level2_projectFiles_selection = input('$ ').split()
    global level2_projectFiles_folderNumber
    level2_projectFiles_folderNumber = 1

    for string in level2_projectFiles_selection:
        if string == '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'PREMIERE PRO', "PROJECT FILES/PREMIERE PRO")
        elif string == '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'DAVINCI RESOLVE', "PROJECT FILES/DAVINCI RESOLVE")
        elif string == '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'MEDIA COMPOSER', "PROJECT FILES/MEDIA COMPOSER")
        elif string == '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'FINAL CUT PRO', "PROJECT FILES/FINAL CUT PRO")
        elif string == '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'SONY VEGAS', "PROJECT FILES/SONY VEGAS")
        elif string == '6': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'HITFILM EXPRESS', "PROJECT FILES/HITFILM EXPRESS")
        elif string == '7': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AFTER EFFECT', "PROJECT FILES/AFTER EFFECTS")
        elif string == '8': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'PHOTOSHOP', "PROJECT FILES/PHOTOSHOP")
        elif string == '9': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'ILLUSTRATOR', "PROJECT FILES/ILLUSTRATOR")
        elif string == '10': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'BLENDER', "PROJECT FILES/BLENDER")
        elif string == '11': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'CINEMA 4D', "PROJECT FILES/CINEMA 4D")
        elif string == '12': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'MAYA', "PROJECT FILES/MAYA")
        elif string == '13': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'NUKE', "PROJECT FILES/NUKE")
        elif string == '14': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'HOUDINI', "PROJECT FILES/HOUDINI")
        elif string == '15': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'ZBRUSH', "PROJECT FILES/ZBRUSH")
        elif string == '16': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AFFINITY DESIGNER', "PROJECT FILES/AFFINITY DESIGNER")
        elif string == '17': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'PRO TOOLS', "PROJECT FILES/PRO TOOLS")
        elif string == '18': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'ABLETON', "PROJECT FILES/ABLETON")
        elif string == '19': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'CUBASE', "PROJECT FILES/CUBASE")
        elif string == '20': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'REAPER', "PROJECT FILES/REAPER")
        elif string == '21': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'IZOTOPE RX', "PROJECT FILES/IZOTOPE RX")
        elif string == '22': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AUDITION', "PROJECT FILES/AUDITION")
        elif string == '23': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'AUDACITY', "PROJECT FILES/AUDACITY")
        elif string == '24': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'LOGIC PRO', "PROJECT FILES/LOGIC PRO")
        elif string == '25': createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, 'FL STUDIO', "PROJECT FILES/FL STUDIO")
        else: createFolder_level2(f'{prefix(level1_project_folderNumber)}PROJECT FILES', level2_projectFiles_folderNumber, string.upper(), f"PROJECT FILES/{string.upper()}")
        level2_projectFiles_folderNumber = level2_projectFiles_folderNumber + 1



def createFolder_level2_assets():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}

    1. LOGOS
    2. GRAPHICS
    3. FONTS
    4. OVERLAYS
    5. REFERENCE

Type custom folder names (separate by SPACE)''')

    level2_assets_selection = input('$ ').split()
    global level2_assets_folderNumber
    level2_assets_folderNumber = 1

    for string in level2_assets_selection:
        if string == '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'LOGOS', "ASSETS/LOGOS")
        elif string == '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'GRAPHICS', "ASSETS/GRAPHICS")
        elif string == '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'FONTS', "ASSETS/FONTS")
        elif string == '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'OVERLAYS', "ASSETS/OVERLAYS")
        elif string == '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, 'REFERENCE', "ASSETS/REFERENCE")
        else: createFolder_level2(f'{prefix(level1_project_folderNumber)}ASSETS', level2_assets_folderNumber, string.upper(), f"ASSETS/{string.upper()}")
        level2_assets_folderNumber = level2_assets_folderNumber + 1



def createFolder_level2_footage():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}
    
    1) RAW
    2) PROXIES
    3) STOCK
    4) VFX
    5) GRADED
          
Type numbers to select or type custom folder names (separate by SPACE)''')

    level2_footage_selection = input('$ ').split()
    global level2_footage_folderNumber
    level2_footage_folderNumber = 1

    for string in level2_footage_selection:
        if string == '1':
            createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'RAW', "FOOTAGE/RAW")
            createFolder_level3_raw()
        elif string == '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'PROXIES', "FOOTAGE/PROXIES")
        elif string == '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'STOCK', "FOOTAGE/STOCK")
        elif string == '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'VFX', "FOOTAGE/VFX")
        elif string == '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, 'GRADED', "FOOTAGE/GRADED")
        else: createFolder_level2(f'{prefix(level1_project_folderNumber)}FOOTAGE', level2_footage_folderNumber, string.upper(), f"FOOTAGE/{string.upper()}")
        level2_footage_folderNumber = level2_footage_folderNumber + 1



def createFolder_level2_audio():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}

    1) PFX
    2) SFX
    3) FOLEY
    4) AMBIANCE
    5) MUSIC
    6) SOUNDTRACK
    7) VOICEOVER
    8) ADR

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_audio_selection = input('$ ').split()
    global level2_audio_folderNumber
    level2_audio_folderNumber = 1

    for string in level2_audio_selection:
        if string == '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'PFX', "AUDIO/PFX")
        elif string == '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'SFX', "AUDIO/SFX")
        elif string == '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'FOLEY', "AUDIO/FOLEY")
        elif string == '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'AMBIANCE', "AUDIO/AMBIANCE")
        elif string == '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'MUSIC', "AUDIO/MUSIC")
        elif string == '6': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'SOUNDTRACK', "AUDIO/SOUNDTRACK")
        elif string == '7': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'VOICEOVER', "AUDIO/VOICEOVER")
        elif string == '8': createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_folderNumber, 'ADR', "AUDIO/ADR")
        else: createFolder_level2(f'{prefix(level1_project_folderNumber)}AUDIO', level2_audio_selection, string.upper(), f"AUDIO/{string.upper()}")
        level2_audio_folderNumber = level2_audio_folderNumber + 1



def createFolder_level2_export():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}

    1) PREVIEWS
    2) MASTER
    3) DELIVERABLES

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_export_selection = input('$ ').split()
    global level2_export_folderNumber
    level2_export_folderNumber = 1

    for string in level2_export_selection:
        if string == '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, 'PREVIEWS', "EXPORT/PREVIEWS")
        elif string == '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, 'MASTER', "EXPORT/MASTER")
        elif string == '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, 'DELIVERABLES', "EXPORT/DELIVERABLES")
        else: createFolder_level2(f'{prefix(level1_project_folderNumber)}EXPORT', level2_export_folderNumber, string.upper(), f"EXPORT/{string.upper()}")
        level2_export_folderNumber = level2_export_folderNumber + 1



def createFolder_level2_documents():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}
    
    1) NOTES
    2) MOODBOARD
    3) STORYBOARD
    4) SCRIPT
    5) PITCH
    6) LEGAL
    7) FINANCIAL

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_documents_selection = input('$ ').split()
    global level2_documents_folderNumber
    level2_documents_folderNumber = 1

    for string in level2_documents_selection:
        if string == '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'NOTES', "DOCUMENTS/NOTES")
        elif string == '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'MOODBOARD', "DOCUMENTS/MOODBOARD")
        elif string == '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'STORYBOARD', "DOCUMENTS/STORYBOARD")
        elif string == '4': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'SCRIPT', "DOCUMENTS/SCRIPT")
        elif string == '5': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'PITCH', "DOCUMENTS/PITCH")
        elif string == '6': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'LEGAL', "DOCUMENTS/LEGAL")
        elif string == '7': createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, 'FINANCIAL', "DOCUMENTS/FINANCIAL")
        else: createFolder_level2(f'{prefix(level1_project_folderNumber)}DOCUMENTS', level2_documents_folderNumber, string.upper(), f"DOCUMENTS/{string.upper()}")
        level2_documents_folderNumber = level2_documents_folderNumber + 1



def createFolder_level2_marketing():
    clearConsole()
    print(f'''Select needed level 2 folders for {level1_folder_name}
    
    1) POSTERS
    2) SOCIAL MEDIA
    3) TRAILERS
    
Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level2_marketing_selection = input('$ ').split()
    global level2_marketing_folderNumber
    level2_marketing_folderNumber = 1

    for string in level2_marketing_selection:
        if string == '1': createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, 'POSTERS', "MARKETING/POSTERS")
        elif string == '2': createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, 'SOCIAL MEDIA', "MARKETING/SOCIAL MEDIA")
        elif string == '3': createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, 'TRAILERS', "MARKETING/TRAILERS")
        else: createFolder_level2(f'{prefix(level1_project_folderNumber)}MARKETING', level2_marketing_folderNumber, string.upper(), f"MARKETING/{string.upper()}")
        level2_marketing_folderNumber = level2_marketing_folderNumber + 1



def createFolder_level2_custom(level2_custom_parent, relative_template_path):
    clearConsole()
    print(f'''Select needed level 2 folders for {level2_custom_parent}

Type custom folder names (separate by SPACE)''')
    
    level2_custom_selection = input('$ ').split()
    global level2_custom_folderNumber
    level2_custom_folderNumber = 1

    for string in level2_custom_selection:
        createFolder_level2(level2_custom_parent, level2_custom_folderNumber, string.upper(), relative_template_path)
        level2_custom_folderNumber = level2_custom_folderNumber + 1



def createFolder_level3(parent_level1:str, parent_level2:str, number:int, name:str, relative_template_path:str):
    level3_folder_name = f'{prefix(number)}{name}'
    level3_folder_directory = os.path.join(projectDirectory, parent_level1, parent_level2, level3_folder_name)

    os.mkdir(level3_folder_directory)
    if initfilm.templates.check(relative_template_path) == True:
        initfilm.templates.copy(relative_template_path, level3_folder_directory, level3_folder_name)



def createFolder_level3_raw():
    clearConsole()
    print(f'''Select needer level 3 folders for {level2_folder_name}
    1) A-CAM
    2) B-CAM
    3) C-CAM
    4) D-CAM
    99) SORT LATER

Type numbers to select or type custom folder names (separate by SPACE)''')
    
    level3_raw_selection = input('$ ').split()
    global level3_raw_folderNumber
    level3_raw_folderNumber = 1

    for string in level3_raw_selection:
        if string == '1': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'A-CAM', "FOOTAGE/RAW/A-CAM")
        elif string == '2': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'B-CAM', "FOOTAGE/RAW/B-CAM")
        elif string == '3': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'C-CAM', "FOOTAGE/RAW/C-CAM")
        elif string == '4': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, 'D-CAM', "FOOTAGE/RAW/D-CAM")
        elif string == '99': createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', 99, 'SORT LATER', "FOOTAGE/RAW/SORT LATER")
        else: createFolder_level3(f'{prefix(level1_project_folderNumber)}FOOTAGE', f'{prefix(level2_footage_folderNumber)}RAW', level3_raw_folderNumber, string.upper(), f"FOOTAGE/RAW/{string.upper()}")
        level3_raw_folderNumber = level3_raw_folderNumber + 1


import sysconfig, os, json



python_sitepackages = sysconfig.get_path("purelib")

if not os.path.exists(f"{python_sitepackages}/initfilm/config.json"):
    with open(f"{python_sitepackages}/initfilm/config_default.json", "r") as config_default:
        config_default = json.load(config_default)
        with open(f"{python_sitepackages}/initfilm/config.json", "w") as config:
            config.seek(0)
            json.dump(config_default, config, indent=4)
            config.truncate()



def set(name:str, parameter:str, value:str):
    """Change values in the config file.
    Args:
        name (str): Name of the config property you want to change. (ex. 'prefix' or 'templates')
        parameter (str): Name of the property parameter you want to change. (ex. 'number_style' or 'path')
        value (str): Value to change the property to. (ex. 'default' or 'dot')"""

    with open(f"{python_sitepackages}/initfilm/config.json", "r+") as file:
        config = json.load(file)
        config[name][parameter] = value
        file.seek(0)
        json.dump(config, file, indent=4)
        file.truncate()



def get(name:str, parameter:str) -> str:
    """Get values from the config file.
    Args:
        name (str): Name of the config property you want to get. (ex. 'prefix' or 'templates')
        parameter (str): Name of the property parameter you want to get. (ex. 'number_style' or 'path')
    Returns:
        value (str): The value of the parameter you defined."""

    with open(f"{python_sitepackages}/initfilm/config.json", "r") as file:
        config = json.load(file)
    return config[name][parameter]



def show():
    """Print current configuration."""

    with open(f"{python_sitepackages}/initfilm/config.json", "r") as file:
        config = json.dumps(json.load(file), indent=2)
    print(f"{python_sitepackages}/initfilm/config.json\n" + config)



def setPrefixVisibility(input:str):
    """Set `prefix` `visibility` parameter to input.
    Args:
        input (str): Value to set `visibility` to."""
    
    match input:
        case "visible" | "hidden":
            set("prefix", "visibility", input)
        case _:
            print("Please specify if you want the prefix to be 'visible' or 'hidden'.")



def setNumberStyle(input:str):
    """Set `prefix` `number_style` parameter to input.
    Args:
        input (str): Value to set `number_style` to."""

    match input:
        case "default" | "double":
            set("prefix", "number_style", input)
        case _:
            print("Invalid number style! Valid options are 'default' or 'double'.")



def setSeparatorStyle(input:str):
    """Set `prefix` `separator_style` parameter to input.
    Args:
        input (str): Value to set `separator_style` to."""

    match input:
        case "dot" | "underscore" | "parenthesis" | "space":
            set("prefix", "separator_style", input)
        case _:
            print("Invalid separator style! Valid options are 'dot', 'underscore', 'parenthesis' or 'space'.")